"""
"""

import argparse
import hivmmer
import pandas as pd
from Bio import Seq

_ambiguous = dict(("".join(sorted(b)), a) for a, b in Seq.IUPAC.IUPACData.ambiguous_dna_values.items())
_frequencies = [0.01, 0.02, 0.05, 0.1, 0.15, 0.2, 0.25, 0.4]

def consensus(codonfile, outfile, min_coverage=1000):
    """
    Write FASTA `outfile` with consensus sequences at varying
    thresholds, using only the variants above `min_coverage`.
    """
    codons = pd.read_csv(codonfile, sep="\t", index_col="hxb2").fillna("")

    # Sum counts by site
    sums = codons.groupby(level=0)["count"].sum()
    sums = sums.to_frame(name="sum")
    codons = codons.join(sums, how="left")

    # Filter out low-coverage sites
    codons = codons[codons["sum"] >= min_coverage]

    # Normalize counts
    codons["frequency"] = codons["count"] * (1.0 / codons["sum"])

    hxb2 = hivmmer.list_hxb2()
    with open(outfile, "w") as f:
        for freq in _frequencies:
            codons = codons[codons["frequency"] >= freq]
            print(">{}".format(freq), file=f)
            seq = []
            for i in hxb2:
                if not i in codons.index:
                    seq.append("---")
                else:
                    for nt in zip(*codons.loc[[i], "codon"].tolist()):
                        seq.append(_ambiguous["".join(sorted(set(nt)))])
            print("".join(seq).strip("-"), file=f)

def _run():

    parser = argparse.ArgumentParser()
    parser.add_argument("-n", "--min-coverage",
                        default=1000,
                        metavar="N",
                        type=int,
                        help="minimum coverage for sites included in the consensus (default: 1000)")
    parser.add_argument("CODONS",
                        help="input tab-separated file with codon frequencies")
    parser.add_argument("FASTA",
                        help="output FASTA file with consensus sequences")
    args = parser.parse_args()

    consensus(args.CODONS, args.FASTA, args.min_coverage)

# vim: expandtab sw=4 ts=4



"""
"""
import math
import pandas as pd
import sys
from Bio import SearchIO
from Bio import Seq
from Bio import SeqIO
from importlib import resources
from itertools import chain

# For more information on how thresholds were estimated,
# see `validation/README.md` in the hivmmer git repo.
thresholds = {
    "env": 18.0,
    "gag": 6.5,
    "nef": 4.4,
    "pol": 6.6,
    "tat": 0.8,
    "vif": 3.1,
    "vpr": 2.7,
    "vpu": 2.3
}

dblengths = {
    "env": 854*5226,
    "gag": 429*5183,
    "nef": 207*5188,
    "pol": 983*2956,
    "tat": 71*3361,
    "vif": 171*3937,
    "vpr": 89*3931,
    "vpu": 53*4573
}

def _load_hxb2(gene):
    """
    Load a pre-computed index that converts HMM position to
    HXB2 coordinates for `gene`.
    """
    with resources.open_binary("hivmmer", "{}.hxb2.tsv".format(gene)) as f:
        return pd.read_csv(f, sep="\t", usecols=["hmm", "hxb2", "ins", "del"], index_col="hmm")

def codons(readfile, hmmerfile, gene):
    """
    """

    dblength  = dblengths[gene]
    threshold = thresholds[gene]
    coords    = _load_hxb2(gene)

    reads = SeqIO.index(readfile, "fasta")
    hmmer = SearchIO.read(hmmerfile, "hmmer3-text")

    counts = dict((hxb2, {}) for hxb2 in range(coords.hxb2.iloc[0], coords.hxb2.iloc[-1] + 1, 3))

    for hit in hmmer.hits:

        # Skip hits that contain stop codons (indicates wrong frame)
        if "*" in chain(hsp.aln[1].seq for hsp in hit.hsps): continue

        id, _, frame = hit.id.rpartition("-")
        count = int(id.partition("-")[2])

        if frame.endswith("'"):
            seq = str(reads[id].seq.reverse_complement())
            offset = int(frame[:-1])
        else:
            seq = str(reads[id].seq)
            offset = int(frame)

        for hsp in hit.hsps:

            if math.log((dblength * hsp.hit_span) / 2**hsp.bitscore) >= threshold: continue

            i    = 0                         # tracks position in the alignment (0-indexed)
            hmm  = hsp.query_start + 1       # tracks position in the HMM reference sequence (1-indexed)
            read = 3*hsp.hit_start + offset  # tracks position in the read sequence (0-indexed)

            # read/reference sequences should have same length in alignment
            n = len(hsp.aln[0].seq)
            assert len(hsp.aln[1].seq) == n

            # alignment should not start or end with an insertion
            assert hsp.aln[0].seq[0] != "."
            assert hsp.aln[0].seq[n-1] != "."

            while i < n:

                aa_frame = []
                codon_frame = []
                hxb2 = coords.loc[hmm, "hxb2"]

                aa = hsp.aln[1].seq[i]
                if aa != "-":
                    aa_frame.append(aa)
                    codon_frame.append(seq[read:read+3])
                    read += 3

                # Extend frame with insertions relative to HMM
                while i < (n-1) and hsp.aln[0].seq[i+1] == ".":
                    aa_frame.append(hsp.aln[1].seq[i+1])
                    codon_frame.append(seq[read:read+3])
                    assert aa_frame[-1] != "-"
                    read += 3
                    i += 1

                # Extend frame with deletions relative to HXB2
                for _ in range(coords.loc[hmm, "del"]):

                    if i < (n-1):
                        aa = hsp.aln[1].seq[i+1]
                        if aa != "-":
                            aa_frame.append(aa)
                            codon_frame.append(seq[read:read+3])
                            read += 3
                            i += 1
                            hmm += 1

                    while i < (n-1) and hsp.aln[0].seq[i+1] == ".":
                        aa_frame.append(hsp.aln[1].seq[i+1])
                        codon_frame.append(seq[read:read+3])
                        assert aa_frame[-1] != "-"
                        read += 3
                        i += 1

                # Assign codons, adjusting coordinates relative to HXB2 insertions
                assert len(aa_frame) == len(codon_frame)
                for _ in range(coords.loc[hmm, "ins"] + 1):
                    # Iterate through the codon frame
                    if codon_frame:
                        codon = codon_frame.pop(0)
                        aa = aa_frame.pop(0)
                        if aa != 'X' and 'N' not in codon:
                            assert str(Seq.translate(codon)) == aa.upper()
                            counts[hxb2][codon] = counts[hxb2].get(codon, 0) + count
                    # Deletions occur when the codon frame is empty
                    else:
                        counts[hxb2][""] = counts[hxb2].get("", 0) + count
                    hxb2 += 3

                i += 1
                hmm += 1

    # output
    lines = []
    for hxb2 in sorted(counts):
        for codon in sorted(counts[hxb2], key=counts[hxb2].get, reverse=True):
            lines.append("\t".join([str(hxb2), codon, str(counts[hxb2][codon])]))
    return lines

# vim: expandtab sw=4 ts=4



import argparse
import sys
from Bio import SeqIO
from operator import itemgetter


def mean_filter(filename, min_length, min_quality, keep={}):
    """
    Filters out Illumina reads in input FASTA file `filename` with
    length < `min_length` and mean quality score < `min_quality`.

    Deduplicates identical sequences.

    Adds distinct sequences to dictionary `keep`, with the sequence
    count as the value.
    """
    for seq in SeqIO.parse(filename, "fastq"):
        if len(seq) > min_length and sum(seq.letter_annotations["phred_quality"]) / len(seq) > min_quality:
            seq = str(seq.seq)
            keep[seq] = keep.get(seq, 0) + 1


def split_filter(filename, min_length, min_quality, keep={}):
    """
    Split reads in input FASTA file `filename` into subsequences with
    quality score > `min_quality` and length > `min_length`.

    Deduplicates identical subsequences.

    Adds distinct subsequences to dictionary `keep`, with the subsequence
    count as the value.
    """
    for seq in SeqIO.parse(filename, "fastq"):
        quality = seq.letter_annotations["phred_quality"]
        seq = str(seq.seq)
        start = 0
        i = 0
        for i, (nt, q) in enumerate(zip(seq, quality)):
            if nt == 'N' or q < min_quality:
                subseq = seq[start:i+1]
                if len(subseq) > min_length:
                    keep[subseq] = keep.get(subseq, 0) + 1
                start = i


def mask_filter(filename, min_quality, keep={}):
    """
    Mask bases as N in sequences from `filename` if quality score
    is < `min_quality`.

    Deduplicates identical subsequences.

    Adds distinct subsequences to dictionary `keep`, with the subsequence
    count as the value.
    """
    for seq in SeqIO.parse(filename, "fastq"):
        quality = seq.letter_annotations["phred_quality"]
        masked = []
        for nt, q in zip(str(seq.seq), quality):
            if q >= min_quality:
                masked.append(nt)
            else:
                masked.append('N')
        masked = "".join(masked)
        keep[masked] = keep.get(masked, 0) + 1


def tofasta(keep, f):
    """
    Writes distinct sequences/counts in dictionary `keep` to FASTA file `f`.
    """
    for i, (seq, n) in enumerate(sorted(keep.items(), key=itemgetter(1), reverse=True)):
        print(">{}-{}".format(i, n), file=f)
        print(seq, file=f)


def _run():

    parser = argparse.ArgumentParser()
    parser.add_argument("-l", "--min-length",
                        default=75,
                        metavar="L",
                        type=int,
                        help="minimum read length to retain")
    parser.add_argument("-q", "--min-quality",
                        default=25,
                        metavar="Q",
                        type=float,
                        help="minimum quality score to retain")
    parser.add_argument("-m", "--mode",
                        choices=["mean", "split"],
                        default="mean",
                        help="filter on mean quality score vs. split into high-quality subsequences")
    parser.add_argument("FASTQ",
                        nargs="+",
                        help="list of FASTQ input files to filter")
    args= parser.parse_args()

    keep = {}
    modes = {"mean": mean_filter, "split": split_filter}

    for fastq in args.FASTQ:
        modes[args.mode](fastq, args.min_length, args.min_quality, keep)

    tofasta(keep, sys.stdout)


# vim: expandtab sw=4 ts=4



"""
"""
import pandas as pd
from importlib import resources

def _load_drm_list():
    """
    Load a pre-packaged list of IAS and Stanford DRMs.
    """
    with resources.open_binary("hivmmer", "drms.csv") as f:
        return pd.read_csv(f)

def drms(aafile, outfile, coverage=10, frequency=0.01):
    """
    """
    # Load the AA table
    aa = pd.read_excel(aafile)
    del aa["hxb2"]

    # Unpivot and remove zero- or low-coverage calls
    aa = aa.melt(id_vars=["region", "position", "coverage"], var_name="variant", value_name="count")
    aa = aa[aa["count"] >= 10]

    # Retain calls above the frequency threshold
    aa["frequency"] = aa["count"] / aa["coverage"]
    aa = aa[aa["frequency"] >= frequency]
    del aa["coverage"]
    del aa["count"]

    # Inner join the DRM list and output
    aa = aa.merge(_load_drm_list(), on=["region", "position", "variant"], how="inner")
    aa.to_csv(outfile, index=False)

# vim: expandtab sw=4 ts=4



"""
"""

import matplotlib
matplotlib.rcParams["font.sans-serif"] = ["Nimbus Sans L", "Helvetica", "Arial"]

import hivmmer
import matplotlib.pyplot as plt
import numpy as np
import os
import pandas as pd
import time
from collections import defaultdict
from datetime import datetime
from matplotlib.ticker import FixedLocator
from subprocess import run

_genes = {
    "gag": 790,
    "pol": 2085,
    "vif": 5041,
    "vpr": 5559,
    "tat": 5831,
    "vpu": 6062,
    "env": 6225,
    "nef": 8797
}
_end = 9417


def plot_coverage(codonfile, outfile, min_coverage=1000):
    """
    Write a PDF plot to `outfile` showing the coverage at each HXB2 position
    based on the codon counts in `codonfile`.
    """
    codons = pd.read_csv(codonfile, sep="\t", index_col="hxb2").fillna("")
    coverage = codons.groupby(level=0)["count"].sum()
    max_coverage = coverage.max()

    # Initialize plot
    fig, ax = plt.subplots(1, 1, figsize=(10, 2))

    # Draw coverage
    xlim = (min(_genes.values()), _end)
    ax.fill_between(xlim, (min_coverage, min_coverage), color="gray")
    ax.fill_between(coverage.index, coverage.values, step="pre", color="k")

    # Setup x axis
    ax.set_xlim(xlim)
    ax.set_xticks(list(_genes.values()))
    ax.set_xticklabels(list(_genes.values()), rotation=90)
    ax2 = ax.twiny()
    ax2.set_xlim(ax.get_xlim())
    ax2.set_xticks(list(_genes.values()))
    ax2.set_xticklabels(list(_genes.keys()), rotation=90)
    ax.grid(axis="x")

    # Setup y axis
    ax.set_ylim(0, max(min_coverage, max_coverage))
    ax.set_yticks([min_coverage, max_coverage])

    # Finalize and write to file
    plt.tight_layout()
    plt.savefig(outfile)
    plt.close()


def plot_coverage_prrt(aafile, outfile, min_coverage=1000):
    """
    Write a PDF plot to `outfile` showing the coverage in the PR/RT/IN
    region of the pol gene.
    """
    aa = pd.read_excel(aafile)

    # Initialize plot
    fig, ax = plt.subplots(1, 1, figsize=(10, 1.5))

    # Serialize the X coordinates for PR, RT, IN
    aa["x"] = aa["position"]
    aa.loc[aa["region"] == "RT", "x"] = aa.loc[aa["region"] == "RT", "position"] + 100
    aa.loc[aa["region"] == "IN", "x"] = aa.loc[aa["region"] == "IN", "position"] + 541
    del aa["region"]
    del aa["position"]

    # Calculate total coverage
    coverage = aa.groupby("x")["coverage"].sum()
    max_coverage = coverage.max()

    # Plot coverage
    xlim = (1, 811)
    ax.fill_between(xlim, (min_coverage, min_coverage), color="gray")
    ax.fill_between(coverage.index, coverage.values, step="pre", color="k")

    # Setup x axis
    xticks = [1, 101, 542]
    xlabels = ["PR", "RT", "IN"]
    ax.set_xlim(*xlim)
    ax.set_xticks(xticks)
    ax.set_xticklabels(xlabels, fontsize=12)
    ax.grid(axis="x")

    # Setup y axis
    ax.set_ylim(0, max(min_coverage, max_coverage))
    ax.set_yticks([min_coverage, max_coverage])

    # Finalize and write to file
    plt.tight_layout()
    plt.savefig(outfile)
    plt.close()


def plot_drms(aafile, drmfile, column, outfile, yticks=[0.1, 1, 5, 10, 20, 50, 100], drmfreq=0.01):
    """
    Write a PDF plot to `outfile` showing DRMs identified by `column`.
    Return a dictionary of DRMs by PI/NRTI/NNRTI/INSTI.
    """
    aa = pd.read_excel(aafile)
    drm = pd.read_csv(drmfile)
    drm = drm[(drm[column] == 1) & (drm["frequency"] >= drmfreq)]

    # Initialize plot
    fig, ax = plt.subplots(1, 1, figsize=(15, 3))

    # Serialize the X coordinates for PR, RT, IN
    aa["x"] = aa["position"]
    aa.loc[aa["region"] == "RT", "x"] = aa.loc[aa["region"] == "RT", "position"] + 100
    aa.loc[aa["region"] == "IN", "x"] = aa.loc[aa["region"] == "IN", "position"] + 541
    del aa["region"]
    del aa["position"]

    drm["x"] = drm["position"]
    drm.loc[drm["region"] == "RT", "x"] = drm.loc[drm["region"] == "RT", "position"] + 100
    drm.loc[drm["region"] == "IN", "x"] = drm.loc[drm["region"] == "IN", "position"] + 541

    # Calculate total coverage
    coverage = aa.groupby("x")["coverage"].sum()

    # Unpivot and remove zero- or low-coverage calls
    del aa["hxb2"]
    aa = aa.melt(id_vars=["x", "coverage"], var_name="variant", value_name="count")
    aa["frequency"] = aa["count"] / aa["coverage"]

    # X axis
    margin = 1.5
    xticks = [1] + list(range(10, 100, 10)) + [101] + list(range(110, 540, 10)) + [542] + list(range(551, 811, 10))
    xlabels = [1] + list(range(10, 100, 10)) + [1] + list(range(10, 440, 10)) + [1] + list(range(10, 269, 10))
    ax.set_xlim(1-margin, 811+margin)
    ax.set_xticks(xticks)
    ax.set_xticklabels(xlabels, fontsize=9, rotation=90)
    ax.set_xlabel("AA Position", size=14)
    ax.xaxis.set_minor_locator(FixedLocator(list(range(1, 811))))
    ax.xaxis.set_tick_params(direction="out")

    # Y axis
    ax.set_ylabel("AA Frequency", size=14)
    ax.set_yscale("log")
    ax.set_ylim(0.9*yticks[0], 1.1*yticks[-1])
    ax.set_yticks(yticks)
    ax.set_yticklabels(['%g%%' % i for i in yticks], size=14)
    ax.get_yaxis().set_tick_params(direction="out")

    # Grid
    ax.grid(True, which="major")
    ax.axvline(x=100, lw=1.0, color="k")
    ax.axvline(x=541, lw=1.0, color="k")

    # Scatterplot
    ax.scatter(aa["x"], 100*aa["frequency"], ec="k", alpha=0.5, marker=".", fc="none")

    # Annotate DRMs
    if len(drm) > 0:
        ax.scatter(drm["x"], 100*drm["frequency"], fc="r", marker=".", ec="none")
        ax.scatter(drm["x"], 100*drm["frequency"], ec="r", marker="o", fc="none")

    # Finalize and write to file
    plt.tight_layout()
    plt.savefig(outfile)
    plt.close()

    drms = defaultdict(list)
    for row in drm.itertuples():
        drms[row.drug].append("".join(map(str, [row.consensus, row.position, row.variant])))
    return dict((drug, ", ".join(drms[drug])) for drug in drms)


def compile(fastq, coveragefile, coverageprrtfile,
            drmsfile, drmifile, sdrmfile,
            drms, drmi, sdrm,
            workdir, outfile):
    """
    Write a PDF report to `outfile` containing coverage, DRM, and SDRM plots.
    """

    name = os.path.basename(fastq[0]).partition("_")[0]

    # Summarize FASTQ input files
    fastq_summary = []
    for f in fastq:
        st = os.stat(f)
        mb = st.st_size / 1048576
        ts = time.strftime("%-d %b %Y", time.localtime(st.st_mtime))
        fastq_summary.append(r"%s (%.1f MB, %s) \\" % (f, mb, ts))
    fastq_summary[-1] = fastq_summary[-1] + "[3pt]"
    fastq_summary = "\n".join(fastq_summary)

    tex = r"""\documentclass{article}
\usepackage[letterpaper, margin=0.5in]{geometry}
\usepackage{graphicx}
\usepackage{fontspec}
\setmainfont{Nimbus Sans L}
\usepackage[small]{titlesec}
\titlespacing*{\section}{0pt}{0pt}{0pt}
\titlespacing*{\subsection}{0pt}{0pt}{0pt}
\setcounter{secnumdepth}{0}

\begin{document}
\pagenumbering{gobble}
\footnotesize

\section{Dataset: %s}
{\em Report generated on %s by hivmmer %s from input:} \\
%s

\subsection{Coverage: Whole Genome}
\includegraphics[width=7.5in]{%s}

\subsection{Coverage: PR/RT/IN}
\includegraphics[width=7.5in]{%s}

\footnotesize
\subsection{DRMs: Stanford}
\textbf{PI}: %s $\;$ \textbf{NRTI}: %s $\;$ \textbf{NNRTI}: %s $\;$ \textbf{INSTI}: %s \\
\includegraphics[width=7.5in]{%s}

\subsection{DRMs: IAS}
\textbf{PI}: %s $\;$ \textbf{NRTI}: %s $\;$ \textbf{NNRTI}: %s $\;$ \textbf{INSTI}: %s \\
\includegraphics[width=7.5in]{%s}

\subsection{SDRMs: Stanford}
\textbf{PI}: %s $\;$ \textbf{NRTI}: %s $\;$ \textbf{NNRTI}: %s $\;$ \textbf{INSTI}: %s \\
\includegraphics[width=7.5in]{%s}
\end{document}
""" % (name, datetime.now().ctime(), hivmmer.__version__, fastq_summary, coveragefile, coverageprrtfile,
       drms.get("PI", "n/a"), drms.get("NRTI", "n/a"), drms.get("NNRTI", "n/a"), drms.get("INSTI", "n/a"),
       drmsfile,
       drmi.get("PI", "n/a"), drmi.get("NRTI", "n/a"), drmi.get("NNRTI", "n/a"), drmi.get("INSTI", "n/a"),
       drmifile,
       sdrm.get("PI", "n/a"), sdrm.get("NRTI", "n/a"), sdrm.get("NNRTI", "n/a"), sdrm.get("INSTI", "n/a"),
       sdrmfile)

    # Write tex to flie
    texfile = os.path.join(workdir, "report.tex")
    with open(texfile, "w") as f:
        f.write(tex.replace("_", "\_"))

    # Run tectonic
    with open(os.path.join(workdir, "tectonic.log"), "w") as f:
        run(["tectonic", texfile], stdout=f, stderr=f, check=True)

    outfile = os.path.abspath(outfile)
    os.rename(os.path.join(workdir, "report.pdf"), outfile)
    return outfile


# vim: expandtab sw=4 ts=4



"""
"""

import numpy as np
import os
import pandas as pd
import sys
from Bio import Seq

aa_header = ["A", "C", "D", "E", "F", "G", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y", "*", "X", "del", "ins"]

ranges = {
    "PR": range(2253, 2550, 3),
    "RT": range(2550, 3870, 3),
    "IN": range(4230, 5037, 3)
}

def aa_table(codonfile, outfile):
    """
    """
    codons = pd.read_csv(codonfile, sep="\t", index_col="hxb2").fillna("")
    subtables = []

    for region in ranges:
        subtable = pd.DataFrame(columns=["region", "position", "coverage"]+aa_header,
                                index=ranges[region]).fillna(0)
        subtable["region"] = region
        subtable["position"] = np.arange(1, len(subtable)+1)
        for hxb2 in subtable.index:
            if hxb2 in codons.index:
                rows = codons.loc[[hxb2]]
                for codon, count in zip(rows["codon"], rows["count"]):
                    if codon == "":
                        subtable.loc[hxb2, "del"] += count
                    elif len(codon) > 3:
                        subtable.loc[hxb2, "ins"] += count
                        aa = str(Seq.translate(codon[:3]))
                        subtable.loc[hxb2, aa] += count
                    else:
                        aa = str(Seq.translate(codon))
                        subtable.loc[hxb2, aa] += count
        subtable["coverage"] = subtable[aa_header].sum(axis=1)
        subtables.append(subtable)

    pd.concat(subtables).to_excel(outfile, index_label="hxb2")

# vim: expandtab sw=4 ts=4



"""
hivmmer - An alignment and variant-calling pipeline for Illumina deep sequencing of
          HIV-1, based on the probabilistic aligner HMMER.

https://github.com/kantorlab/hivmmer

Please cite:

Howison M, Coetzer M, Kantor R. 2019. Measurement error and variant-calling in
deep Illumina sequencing of HIV. Bioinformatics 35(12): 2029-2035.
doi:10.1093/bioinformatics/bty919
"""

import os
import pandas as pd
from importlib import resources

import hivmmer.filter
import hivmmer.report
from .codons import codons
from .consensus import consensus
from .drms import drms
from .table import aa_table
from .translate import translate, translate_unambiguous

__version__ = resources.read_text("hivmmer", "VERSION").strip()

genes = ("gag", "pol", "vif", "vpr", "tat", "vpu", "env", "nef")

def copy_hmms(dst):
    """
    Copy the prepackaged HMM files for whole genome HIV alignment to the `dst` directory.
    """
    for gene in genes:
        for ext in ("h3f", "h3i", "h3m", "h3p"):
            fname = "{}.hmm.{}".format(gene, ext)
            with open(os.path.join(dst, fname), "wb") as f1:
                with resources.open_binary("hivmmer", fname) as f2:
                    f1.write(f2.read())

def list_hxb2():
    """
    Returns the ordered list of HXB2 coordinates in the prepackaged HMM files.
    """
    hxb2 = []
    for gene in genes:
        with resources.open_binary("hivmmer", "{}.hxb2.tsv".format(gene)) as f:
            hxb2 += pd.read_csv(f, sep="\t", usecols=["hxb2"]).hxb2.tolist()
    return hxb2

# vim: expandtab sw=4 ts=4



import argparse
import sys
from Bio import Seq
from Bio import SeqIO


def translate(filename, out=sys.stdout, log=sys.stderr):
    """
    Translate nucleotide sequences in FASTA file `filename` to all six possible
    frames.

    Write amino acid sequences to FASTA file `out`, with the frame number
    appended to the sequence header.

    Log summary statistics to file `log`.
    """

    nskipped = 0

    for n, record in enumerate(SeqIO.parse(filename, "fasta")):

        seq = str(record.seq)

        if 'N' in seq:
            nskipped += 1
            continue

        for i in range(3):
            j = 3 * ((len(seq) - i) // 3) + i
            print(">%s-%d" % (record.id, i), file=out)
            print(Seq.translate(seq[i:j]), file=out)

        seq = str(record.seq.reverse_complement())

        for i in range(3):
            j = 3 * ((len(seq) - i) // 3) + i
            print(">%s-%d'" % (record.id, i), file=out)
            print(Seq.translate(seq[i:j]), file=out)

    print("nreads", n, file=log)
    print("nskipped (N)", nskipped, file=log)


def translate_unambiguous(filename, out=sys.stdout, log=sys.stderr, fraction=0.9):
    """
    Translate nucleotide sequences in FASTA file `filename` to all six possible
    frames.

    Only keep translated sequences with > `fraction` of unambigous amino acids.

    Write amino acid sequences to FASTA file `out`, with the frame number
    appended to the sequence header.

    Log summary statistics to file `log`.
    """

    nskipped = 0

    for n, record in enumerate(SeqIO.parse(filename, "fasta")):

        seq = str(record.seq)

        for i in range(3):
            j = 3 * ((len(seq) - i) // 3) + i
            tseq = Seq.translate(seq[i:j])
            if sum(aa != 'X' for aa in tseq) / len(tseq) > fraction:
                print(">%s-%d" % (record.id, i), file=out)
                print(tseq, file=out)

        seq = str(record.seq.reverse_complement())

        for i in range(3):
            j = 3 * ((len(seq) - i) // 3) + i
            tseq = Seq.translate(seq[i:j])
            if sum(aa != 'X' for aa in tseq) / len(tseq) > fraction:
                print(">%s-%d'" % (record.id, i), file=out)
                print(tseq, file=out)

    print("nreads", n, file=log)


def _run():

    parser = argparse.ArgumentParser()
    parser.add_argument("FASTA",
                        nargs=1,
                        help="path to nucleotide FASTA file, or '-' to read from stdin")
    args = parser.parse_args()

    fasta = args.FASTA[0]
    if fasta == '-':
        translate(sys.stdin, args.allow_stop_codons)
    else:
        with open(fasta) as f:
            translate(f)


# vim: expandtab sw=4 ts=4



from setuptools import setup, find_packages

with open("hivmmer/VERSION") as f:
    version = f.read().strip()

setup(
    name="hivmmer",
    version=version,
    author="Mark Howison",
    author_email="mhowison@ripl.org",
    url="https://github.com/kantorlab/hivmmer",
    description="""
        An alignment and variant-calling pipeline for Illumina deep sequencing of
        HIV-1, based on the probabilistic aligner HMMER.""",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Topic :: Scientific/Engineering :: Bio-Informatics"],
    provides=["hivmmer"],
    install_requires=["BioPython>=1.69", "matplotlib>=3.1.1", "numpy>=1.13.0", "openpyxl", "pandas>=0.22.0", "xlrd"],
    packages=find_packages(),
    package_data={"hivmmer": ["VERSION", "*.csv", "*.hmm.*", "*.tsv"]},
    scripts=["scripts/hivmmer"],
    entry_points={
        "console_scripts": ["hivmmer-consensus=hivmmer.consensus:_run",
                            "hivmmer-filter=hivmmer.filter:_run",
                            "hivmmer-translate=hivmmer.translate:_run"]
    }
)
